@@ -15,10 +15,14 @@
  * Modified to handle round joins as well (based on common code),
  *                           Linas, March 1993
  * work around OpenGL's lack of support for concave polys, June 1994
+ *
+ * Copyright (C) 1991,1993,1994,2003 Linas Vepstas <linas@linas.org>
  */
 
 
-#include <malloc.h>
+#if defined(_WIN32) && !defined(__clang__)
+# include <malloc.h>
+#endif
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -40,6 +44,8 @@ typedef void (*gleCapCallback) (int iloop,
                                 double norms[][3], 
                                 int frontwards);
 
+#define INVALID_BUG_NEEDS_FIXING 0
+
 #ifdef NONCONCAVE_CAPS
 
 /* ============================================================ */
@@ -128,9 +134,9 @@ draw_cut_style_cap_callback (int iloop,
 #ifdef OPENGL_10
    GLUtriangulatorObj *tobj;
    tobj = gluNewTess ();
-   gluTessCallback (tobj, GLU_BEGIN, glBegin);
-   gluTessCallback (tobj, GLU_VERTEX, glVertex3dv);
-   gluTessCallback (tobj, GLU_END, glEnd);
+   gluTessCallback (tobj, GLU_BEGIN, (_GLUfuncptr) glBegin);
+   gluTessCallback (tobj, GLU_VERTEX, (_GLUfuncptr) glVertex3dv);
+   gluTessCallback (tobj, GLU_END, (_GLUfuncptr) glEnd);
 #endif /* OPENGL_10 */
 
    if (face_color != NULL) C3F (face_color);
@@ -302,13 +308,13 @@ draw_fillet_triangle_plain
    if (front_color != NULL) C3F (front_color);
    BGNTMESH (-5, 0.0);
    if (face) {
-      V3F (va, -1, FILLET);
-      V3F (vb, -1, FILLET);
+      V3F (va, INVALID_BUG_NEEDS_FIXING, FILLET);
+      V3F (vb, INVALID_BUG_NEEDS_FIXING, FILLET);
    } else {
-      V3F (vb, -1, FILLET);
-      V3F (va, -1, FILLET);
+      V3F (vb, INVALID_BUG_NEEDS_FIXING, FILLET);
+      V3F (va, INVALID_BUG_NEEDS_FIXING, FILLET);
    }
-   V3F (vc, -1, FILLET);
+   V3F (vc, INVALID_BUG_NEEDS_FIXING, FILLET);
    ENDTMESH ();
 
 }
@@ -347,27 +353,27 @@ static void draw_fillet_triangle_n_norms
    if (__TUBE_DRAW_FACET_NORMALS) {
       N3F_D (na);
       if (face) {
-         V3F (va, -1, FILLET);
-         V3F (vb, -1, FILLET);
+         V3F (va, INVALID_BUG_NEEDS_FIXING, FILLET);
+         V3F (vb, INVALID_BUG_NEEDS_FIXING, FILLET);
       } else {
-         V3F (vb, -1, FILLET);
-         V3F (va, -1, FILLET);
+         V3F (vb, INVALID_BUG_NEEDS_FIXING, FILLET);
+         V3F (va, INVALID_BUG_NEEDS_FIXING, FILLET);
       }
-      V3F (vc, -1, FILLET);
+      V3F (vc, INVALID_BUG_NEEDS_FIXING, FILLET);
    } else {
       if (face) {
          N3F_D (na);
-         V3F (va, -1, FILLET);
+         V3F (va, INVALID_BUG_NEEDS_FIXING, FILLET);
          N3F_D (nb);
-         V3F (vb, -1, FILLET);
+         V3F (vb, INVALID_BUG_NEEDS_FIXING, FILLET);
       } else {
          N3F_D (nb);
-         V3F (vb, -1, FILLET);
+         V3F (vb, INVALID_BUG_NEEDS_FIXING, FILLET);
          N3F_D (na);
-         V3F (va, -1, FILLET);
+         V3F (va, INVALID_BUG_NEEDS_FIXING, FILLET);
          N3F_D (nb);
       }
-      V3F (vc, -1, FILLET);
+      V3F (vc, INVALID_BUG_NEEDS_FIXING, FILLET);
    }
    ENDTMESH ();
 
@@ -396,7 +402,7 @@ static void draw_fillets_and_join_plain
    int save_style = 0;
    int was_trimmed = FALSE;
 
-   cap_loop = (double *) malloc ((ncp+3)*3*sizeof (double));
+   cap_loop = (double *) malloc (((size_t)ncp+3)*3*sizeof (double));
    
    /*
     * If the first point is trimmed, keep going until one
@@ -613,7 +619,7 @@ draw_fillets_and_join_n_norms
    int was_trimmed = FALSE;
 
    save_style = gleGetJoinStyle ();
-   cap_loop = (double *) malloc ((ncp+3)*3*2*sizeof (double));
+   cap_loop = (double *) malloc (((size_t)ncp+3)*3*2*sizeof (double));
    norm_loop = cap_loop + (ncp+3)*3;
    
    /* 
@@ -837,7 +843,7 @@ extrusion_round_or_cut_join (int ncp,	/* number of contour points */
                            gleDouble up[3],	/* up vector for contour */
                            int npoints,		/* numpoints in poly-line */
                            gleDouble point_array[][3],	/* polyline */
-                           float color_array[][3],	/* color of polyline */
+                           gleColor color_array[],	/* color of polyline */
                            gleDouble xform_array[][2][3])   /* 2D contour xforms */
 {
    int i, j;
@@ -904,9 +910,9 @@ extrusion_round_or_cut_join (int ncp,	/* number of contour points */
    neg_z[2] = 1.0;
 
    /* malloc the data areas that we'll need to store the end-caps */
-   mem_anchor = malloc (4 * 3*ncp*sizeof(gleDouble)
-                      + 2 * 3*ncp*sizeof(double)
-                      + 2 * 1*ncp*sizeof(int));
+   mem_anchor = malloc (4 * 3*(size_t)ncp*sizeof(gleDouble)
+                      + 2 * 3*(size_t)ncp*sizeof(double)
+                      + 2 * 1*(size_t)ncp*sizeof(int));
    front_norm = (double *) mem_anchor;
    back_norm = front_norm + 3*ncp;
    front_loop = (gleDouble *) (back_norm + 3*ncp);
@@ -1241,7 +1247,7 @@ extrusion_round_or_cut_join (int ncp,	/* number of contour points */
          if (__TUBE_DRAW_CAP) {
             if (color_array != NULL) C3F (color_array[inext-1]);
             draw_angle_style_front_cap (ncp, bisector_0, 
-                                       (gleDouble (*)[3]) front_loop);
+                                       (gleVector *) front_loop);
          }
       }
       /* v^v^v^v^v^v^v^v^v  END END CAPS v^v^v^v^v^v^v^v^v^v^v^v */
@@ -1284,7 +1290,7 @@ extrusion_round_or_cut_join (int ncp,	/* number of contour points */
             if (__TUBE_DRAW_CAP) {
                if (color_array != NULL) C3F (color_array[inext]);
                draw_angle_style_back_cap (ncp, bisector_1, 
-                                           (gleDouble (*)[3]) back_loop);
+                                           (gleVector *) back_loop);
                cap_callback = null_cap_callback;
             }
          } else {
