@@ -12,10 +12,14 @@
  * added normal vectors Linas, October 1991
  * "code complete" (that is, I'm done), Linas Vepstas, October 1991
  * work around OpenGL's lack of support for concave polys, June 1994
+ *
+ * Copyright (C) 1991,1994,2003 Linas Vepstas <linas@linas.org>
  */
 
 
-#include <malloc.h>
+#if defined(_WIN32) && !defined(__clang__)
+# include <malloc.h>
+#endif
 #include <stdlib.h>
 #include <math.h>
 #include <string.h>	/* for the memcpy() subroutine */
@@ -31,6 +35,7 @@
 #include "segment.h"
 
 /* ============================================================ */
+#ifndef COLOR_SIGNATURE
 /* 
  * The following routine is, in principle, very simple:
  * all that it does is normalize the up vector, and makes
@@ -144,11 +149,11 @@ void draw_raw_style_end_cap (int ncp,		/* number of contour points */
 
 #ifdef OPENGL_10
    /* malloc the @#$%^&* array that OpenGL wants ! */
-   pts = (double *) malloc (3*ncp*sizeof(double));
+   pts = (double *) malloc (3*(size_t)ncp*sizeof(double));
    tobj = gluNewTess ();
-   gluTessCallback (tobj, GLU_BEGIN, glBegin);
-   gluTessCallback (tobj, GLU_VERTEX, glVertex3dv);
-   gluTessCallback (tobj, GLU_END, glEnd);
+   gluTessCallback (tobj, GLU_BEGIN, (_GLUfuncptr)glBegin);
+   gluTessCallback (tobj, GLU_VERTEX, (_GLUfuncptr)glVertex3dv);
+   gluTessCallback (tobj, GLU_END, (_GLUfuncptr)glEnd);
    gluBeginPolygon (tobj);
 
       /* draw the loop counter clockwise for the front cap */
@@ -176,7 +181,7 @@ void draw_raw_style_end_cap (int ncp,		/* number of contour points */
 #endif /* OPENGL_10 */
 }
 
-
+#endif /* COLOR_SIGNATURE */
 /* ============================================================ */
 /* This routine does what it says: It draws a counter-clockwise cap 
  * from a 3D contour loop list
@@ -206,9 +211,9 @@ draw_front_contour_cap (int ncp,	/* number of contour points */
 
 #ifdef OPENGL_10
    tobj = gluNewTess ();
-   gluTessCallback (tobj, GLU_BEGIN, glBegin);
-   gluTessCallback (tobj, GLU_VERTEX, glVertex3dv);
-   gluTessCallback (tobj, GLU_END, glEnd);
+   gluTessCallback (tobj, GLU_BEGIN, (_GLUfuncptr)glBegin);
+   gluTessCallback (tobj, GLU_VERTEX, (_GLUfuncptr)glVertex3dv);
+   gluTessCallback (tobj, GLU_END, (_GLUfuncptr)glEnd);
    gluBeginPolygon (tobj);
 
    for (j=0; j<ncp; j++) {
@@ -252,9 +257,9 @@ draw_back_contour_cap (int ncp,	/* number of contour points */
 
 #ifdef OPENGL_10
    tobj = gluNewTess ();
-   gluTessCallback (tobj, GLU_BEGIN, glBegin);
-   gluTessCallback (tobj, GLU_VERTEX, glVertex3dv);
-   gluTessCallback (tobj, GLU_END, glEnd);
+   gluTessCallback (tobj, GLU_BEGIN, (_GLUfuncptr)glBegin);
+   gluTessCallback (tobj, GLU_VERTEX, (_GLUfuncptr)glVertex3dv);
+   gluTessCallback (tobj, GLU_END, (_GLUfuncptr)glEnd);
    gluBeginPolygon (tobj);
 
    /* draw the end cap */
@@ -319,7 +324,6 @@ draw_raw_segment_plain (int ncp,		/* number of contour points */
       /* draw the back cap */
       draw_raw_style_end_cap (ncp, contour, -len, FALSE);
    }
-
 }
 
 /* ============================================================ */
@@ -332,7 +336,7 @@ draw_raw_segment_plain (int ncp,		/* number of contour points */
 static void 
 draw_raw_segment_color (int ncp,		/* number of contour points */
                              gleDouble contour[][2],	/* 2D contour */
-                             float color_array[][3],	/* color of polyline */
+                             gleColor color_array[],	/* color of polyline */
                              int inext,
                              gleDouble len)
 
@@ -465,7 +469,7 @@ draw_raw_segment_edge_n (int ncp,	/* number of contour points */
 static void 
 draw_raw_segment_c_and_edge_n (int ncp,	/* number of contour points */
                              gleDouble contour[][2],	/* 2D contour */
-                             float color_array[][3],	/* color of polyline */
+                             gleColor color_array[],	/* color of polyline */
                              gleDouble cont_normal[][2],/* 2D normal vecs */
                              int inext,
                              gleDouble len)
@@ -638,7 +642,7 @@ draw_raw_segment_facet_n (int ncp,	/* number of contour points */
 static void 
 draw_raw_segment_c_and_facet_n (int ncp,	/* number of contour points */
                              gleDouble contour[][2],	/* 2D contour */
-                             float color_array[][3],	/* color of polyline */
+                             gleColor color_array[],	/* color of polyline */
                              gleDouble cont_normal[][2],/* 2D normal vecs */
                              int inext,
                              gleDouble len)
@@ -759,7 +763,7 @@ void extrusion_raw_join (int ncp,		/* number of contour points */
                          gleDouble up[3],	/* up vector for contour */
                          int npoints,		/* numpoints in poly-line */
                          gleDouble point_array[][3],	/* polyline */
-                         float color_array[][3],	/* color of polyline */
+                         gleColor color_array[],	/* color of polyline */
                          gleDouble xform_array[][2][3])   /* 2D contour xforms */
 {
    int i, j;
@@ -784,7 +788,7 @@ void extrusion_raw_join (int ncp,		/* number of contour points */
 
    /* alloc loop arrays if needed */
    if (! no_xform) {
-      mem_anchor = malloc (4 * ncp * 3 * sizeof(gleDouble));
+      mem_anchor = malloc (4 * (size_t)ncp * 3 * sizeof(gleDouble));
       front_loop = (gleDouble *) mem_anchor;
       back_loop = front_loop + 3*ncp;
       front_norm = back_loop + 3*ncp;
