@@ -11,10 +11,14 @@
  * AUTHOR:
  * written by Linas Vepstas August/September 1991
  * added polycone, February 1993
+ *
+ * Copyright (C) 1991,1993,2003 Linas Vepstas <linas@linas.org>
  */
 
 
-#include <malloc.h>
+#if defined(_WIN32) && !defined(__clang__)
+# include <malloc.h>
+#endif
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>	/* for the memcpy() subroutine */
@@ -27,6 +31,7 @@
 #include "intersect.h"
 
 /* ============================================================ */
+#ifndef COLOR_SIGNATURE
 /* The routine below  determines the type of join style that will be
  * used for tubing. */
 
@@ -42,6 +47,7 @@ int gleGetJoinStyle (void)
    return (extrusion_join_style);
 }
 
+#endif /* COLOR_SIGNATURE */
 /* ============================================================ */
 /*
  * draw a general purpose extrusion 
@@ -53,7 +59,7 @@ void gleSuperExtrusion (int ncp,               /* number of contour points */
                 gleDouble up[3],           /* up vector for contour */
                 int npoints,           /* numpoints in poly-line */
                 gleDouble point_array[][3],        /* polyline */
-                float color_array[][3],        /* color of polyline */
+                gleColor color_array[],        /* color of polyline */
                 gleDouble xform_array[][2][3])   /* 2D contour xforms */
 {   
    INIT_GC();
@@ -103,7 +109,7 @@ void gleExtrusion (int ncp,               /* number of contour points */
                 gleDouble up[3],           /* up vector for contour */
                 int npoints,           /* numpoints in poly-line */
                 gleDouble point_array[][3],        /* polyline */
-                float color_array[][3])        /* color of polyline */
+                gleColor color_array[])        /* color of polyline */
 {   
    gleSuperExtrusion (ncp, contour, cont_normal, up,
                     npoints,
@@ -118,7 +124,7 @@ void gleExtrusion (int ncp,               /* number of contour points */
 static void 
 gen_polycone (int npoints,
                gleDouble point_array[][3],
-               float color_array[][3],
+               gleColor color_array[],
                gleDouble radius,
                gleDouble xform_array[][2][3])
 {
@@ -182,7 +188,7 @@ gen_polycone (int npoints,
 
 void glePolyCylinder (int npoints,
                    gleDouble point_array[][3],
-                   float color_array[][3],
+                   gleColor color_array[],
                    gleDouble radius)
 {
    gen_polycone (npoints, point_array, color_array, radius, NULL);
@@ -192,14 +198,14 @@ void glePolyCylinder (int npoints,
 
 void glePolyCone (int npoints,
                gleDouble point_array[][3],
-               float color_array[][3],
+               gleColor color_array[],
                gleDouble radius_array[])
 {
    gleAffine * xforms;
    int j;
 
    /* build 2D affine matrices from radius array */
-   xforms = (gleAffine *) malloc (npoints * sizeof(gleAffine));
+   xforms = (gleAffine *) malloc ((size_t)npoints * sizeof(gleAffine));
    for (j=0; j<npoints; j++) {
       AVAL(xforms,j,0,0) = radius_array[j];
       AVAL(xforms,j,0,1) = 0.0;
@@ -222,7 +228,7 @@ void gleTwistExtrusion (int ncp,         /* number of contour points */
                 gleDouble up[3],           /* up vector for contour */
                 int npoints,           /* numpoints in poly-line */
                 gleDouble point_array[][3],        /* polyline */
-                float color_array[][3],        /* color of polyline */
+                gleColor color_array[],        /* color of polyline */
                 gleDouble twist_array[])   /* countour twists (in degrees) */
 
 {
@@ -233,7 +239,7 @@ void gleTwistExtrusion (int ncp,         /* number of contour points */
    gleAffine *xforms;
 
    /* build 2D affine matrices from radius array */
-   xforms = (gleAffine *) malloc (npoints * sizeof(gleAffine));
+   xforms = (gleAffine *) malloc ((size_t)npoints * sizeof(gleAffine));
 
    for (j=0; j<npoints; j++) {
       angle = (M_PI/180.0) * twist_array[j];
@@ -260,6 +266,7 @@ void gleTwistExtrusion (int ncp,         /* number of contour points */
 }
 
 /* ============================================================ */
+#ifndef COLOR_SIGNATURE
 /* 
  * The spiral primitive forms the basis for the helicoid primitive.
  *
@@ -314,11 +321,11 @@ void gleSpiral (int ncp,               /* number of contour points */
    npoints = (int) ((((double) __TESS_SLICES) /360.0) * fabs(sweepTheta)) + 4;
 
    if (startXform == NULL) {
-      mem_anchor = malloc (3*npoints * sizeof (gleDouble));
+      mem_anchor = malloc (3*(size_t)npoints * sizeof (gleDouble));
       pts = (gleDouble *) mem_anchor;
       xforms = NULL;
    } else {
-      mem_anchor = malloc ((1+2)* 3*npoints * sizeof (gleDouble));
+      mem_anchor = malloc ((1+2)* 3*(size_t)npoints * sizeof (gleDouble));
       pts = (gleDouble *) mem_anchor;
       xforms = (gleAffine *) (pts + 3*npoints);
    }
@@ -734,8 +741,8 @@ void gleScrew (int ncp,
    numsegs = (int) fabs (twist / 18.0) + 4;
 
    /* malloc the extrusion array and the twist array */
-   path = (gleVector *) malloc (numsegs * sizeof (gleVector));
-   twarr = (gleDouble *) malloc (numsegs * sizeof (gleDouble));
+   path = (gleVector *) malloc ((size_t)numsegs * sizeof (gleVector));
+   twarr = (gleDouble *) malloc ((size_t)numsegs * sizeof (gleDouble));
 
    /* fill in the extrusion array and the twist array uniformly */
    delta = (endz-startz) / ((gleDouble) (numsegs-3));
@@ -757,4 +764,5 @@ void gleScrew (int ncp,
    free (twarr);
 }
 
+#endif /* COLOR_SIGNATURE */
 /* ============================================================ */
